use galactica_content::{Content, FactionHandle, OutfitHandle, ShipHandle, SystemHandle};
use galactica_galaxy::{ship::ShipPersonality, Galaxy, GxShipHandle};
use galactica_util::timing::Timing;
use std::time::Instant;
use winit::event::{ElementState, MouseButton, MouseScrollDelta, TouchPhase, VirtualKeyCode};

use crate::camera::Camera;
use crate::inputstatus::InputStatus;
use galactica_render::RenderInput;
use galactica_systemsim::{objects::ShipControls, util, ParticleBuilder, StepResources, SystemSim};

pub struct Game {
	input: InputStatus,
	last_update: Instant,
	player: GxShipHandle,
	paused: bool,
	time_scale: f32,
	start_instant: Instant,
	camera: Camera,

	ct: Content,
	gx: Galaxy,

	systemsim: SystemSim,

	new_particles: Vec<ParticleBuilder>,
	timing: Timing,
}

impl Game {
	pub fn new(ct: Content) -> Self {
		let mut galaxy = Galaxy::new(&ct);

		let player = galaxy.create_ship(
			&ct,
			ShipHandle { index: 0 },
			FactionHandle { index: 0 },
			ShipPersonality::Player,
			&SystemHandle { index: 0 },
		);
		let s = galaxy.get_ship_mut(player).unwrap();
		s.add_outfit(&ct.get_outfit(OutfitHandle { index: 0 }));
		s.add_outfit(&ct.get_outfit(OutfitHandle { index: 1 }));
		s.add_outfit(&ct.get_outfit(OutfitHandle { index: 2 }));

		let a = galaxy.create_ship(
			&ct,
			ShipHandle { index: 0 },
			FactionHandle { index: 1 },
			ShipPersonality::Dummy,
			&SystemHandle { index: 0 },
		);
		let s = galaxy.get_ship_mut(a).unwrap();
		s.add_outfit(&ct.get_outfit(OutfitHandle { index: 0 }));
		s.add_outfit(&ct.get_outfit(OutfitHandle { index: 1 }));
		s.add_outfit(&ct.get_outfit(OutfitHandle { index: 2 }));

		let a = galaxy.create_ship(
			&ct,
			ShipHandle { index: 0 },
			FactionHandle { index: 0 },
			ShipPersonality::Point,
			&SystemHandle { index: 0 },
		);
		let s = galaxy.get_ship_mut(a).unwrap();
		s.add_outfit(&ct.get_outfit(OutfitHandle { index: 0 }));
		s.add_outfit(&ct.get_outfit(OutfitHandle { index: 1 }));
		s.add_outfit(&ct.get_outfit(OutfitHandle { index: 2 }));

		let physics = SystemSim::new(&ct, &galaxy, SystemHandle { index: 0 });

		Game {
			last_update: Instant::now(),
			input: InputStatus::new(),
			player,
			start_instant: Instant::now(),

			camera: Camera {
				pos: (0.0, 0.0).into(),
				zoom: 500.0,
				aspect: 1.0,
			},
			//system: object::System::new(&ct, SystemHandle { index: 0 }),
			paused: false,
			time_scale: 1.0,
			systemsim: physics,
			gx: galaxy,
			ct,
			new_particles: Vec::new(),
			timing: Timing::new(),
		}
	}

	pub fn set_camera_aspect(&mut self, v: f32) {
		self.camera.aspect = v
	}

	pub fn process_key(&mut self, state: &ElementState, key: &VirtualKeyCode) {
		self.input.process_key(state, key)
	}

	pub fn process_click(&mut self, state: &ElementState, key: &MouseButton) {
		self.input.process_click(state, key)
	}

	pub fn process_scroll(&mut self, delta: &MouseScrollDelta, phase: &TouchPhase) {
		self.input.process_scroll(delta, phase)
	}

	pub fn set_paused(&mut self, pause: bool) {
		if pause {
			self.paused = true;
			self.input.release_all()
		} else {
			self.paused = false;
		}
	}

	pub fn update(&mut self) {
		let t: f32 = self.last_update.elapsed().as_secs_f32() * self.time_scale;

		self.timing.start_frame();
		self.timing.start_galaxy();
		self.gx.step(t);
		self.timing.mark_galaxy();

		self.systemsim.step(StepResources {
			player: self.player,
			player_controls: ShipControls {
				left: self.input.key_left,
				right: self.input.key_right,
				thrust: self.input.key_thrust,
				guns: self.input.key_guns,
			},
			ct: &self.ct,
			gx: &mut self.gx,
			particles: &mut self.new_particles,
			timing: &mut self.timing,
			t,
		});

		if self.input.v_scroll != 0.0 {
			self.camera.zoom = (self.camera.zoom + self.input.v_scroll)
				.clamp(self.ct.get_config().zoom_min, self.ct.get_config().zoom_max);
			self.input.v_scroll = 0.0;
		}

		self.camera.pos = {
			let o = self.systemsim.get_ship(self.player).unwrap();
			let r = self.systemsim.get_rigid_body(o.rigid_body).unwrap();
			util::rigidbody_position(r)
		};

		self.last_update = Instant::now();
		self.timing.mark_frame();
	}

	pub fn get_frame_state(&mut self) -> RenderInput {
		RenderInput {
			camera_pos: self.camera.pos,
			camera_zoom: self.camera.zoom,
			current_time: self.start_instant.elapsed().as_secs_f32(),
			ct: &self.ct,
			systemsim: &self.systemsim, // TODO: maybe system should be stored here?
			particles: &mut self.new_particles,
			player_data: self.player,
			gx: &self.gx,
			current_system: SystemHandle { index: 0 },
			timing: &mut self.timing,
		}
	}

	pub fn get_content(&self) -> &Content {
		&self.ct
	}
}
